/*
Resource list:
1  = query
2  = temp-table
3  = fill-in
4  = buffer
5  = persistent procedure
6  = x-document
7  = x-noderef
8  = socket
9  = server-socket
10 = server
11 = button
12 = browse
13 = soap-header
14 = soap-header-entryref
15 = sax-attributes
16 = sax-reader
17 = sax-writer

check how resources get deleted when:
A. dynamic resources
   1. d   = created in external proc and saved in shared vars
   2. cd  = created in internal proc and saved in shared vars
   3. fcd = created in function and saved in shared vars
   4. pd  = created in internal proc and exposed via output params
   5. fpd = created in function and exposed via output param
Note that 5 (persistent proc handle) is the same for both static and dynamic. 
B. static resources
   1. s   = created in external proc and saved in shared vars
   2. cs  = created in internal proc and saved in shared vars
   3. fcs = created in function and saved in shared vars
   4. ps  = created in internal proc and exposed via output params
   5. fps = created in function and exposed via output param
Note that 2 is the same in all cases, as a TEMP-TABLE can't be defined in an internal proc/function.
Note that 5 (persistent proc handle) is the same for both static and dynamic. 
Note that 6,7,8,9,10,13,14,15,16,17 are dynamic as they don't have static counterparts

Tested cases and found rules:
I. procedure is ran persistent
   A. dynamic resources
      1. state of resources created by external program and saved in shared vars.
         cases: 001a
         - valid:   all
         - invalid: none
      2. state of resources created by an internal procedure exposed via shared vars. 
         cases: 001b
         - valid:   all
         - invalid: none
      3. state of resources created by a function and exposed via shared vars. 
         cases: 001c
         - valid:   all
         - invalid: none
      4. state of resources created by an internal procedure exposed via output params. 
         cases: 003a
         - valid:   all
         - invalid: none
      5. state of resources created by a function and exposed via output params. 
         cases: 003b
         - valid:   all
         - invalid: none
   B. static resources
      1. state of resources created by external program and saved in shared vars.
         cases: 005a
         - valid:   all
         - invalid: none
      2. state of resources created by an internal procedure exposed via shared vars. 
         cases: 005b
         - valid:   2*, 5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 3, 4, 11, 12
      3. state of resources created by a function and exposed via shared vars. 
         cases: 005c
         - valid:   2*, 5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 3, 4, 11, 12
      4. state of resources created by an internal procedure exposed via output params. 
         cases: 007a
         - valid:   2*, 5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 3, 4, 11, 12
      5. state of resources created by a function and exposed via output params. 
         cases: 007b
         - valid:   2*, 5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 3, 4, 11, 12

II. persistent procedure is deleted
   A. dynamic resources
      1. state of resources created by external program and saved in shared vars.
         cases: 009a
         - valid:   all
         - invalid: none
      2. state of resources created by an internal procedure exposed via shared vars. 
         cases: 009b
         - valid:   all
         - invalid: none
      3. state of resources created by a function and exposed via shared vars. 
         cases: 009c
         - valid:   all
         - invalid: none
      4. state of resources created by an internal procedure exposed via output params. 
         cases: 011a
         - valid:   all
         - invalid: none
      5. state of resources created by a function and exposed via output params. 
         cases: 011b
         - valid:   all
         - invalid: none
   B. static resources
      1. state of resources created by external program and saved in shared vars.
         cases: 013a
         - valid:   5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 2, 3, 4, 11, 12
      2. state of resources created by an internal procedure exposed via shared vars. 
         cases: 013b
         - valid:   5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 2, 3, 4, 11, 12
      3. state of resources created by a function and exposed via shared vars. 
         cases: 013c
         - valid:   5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 2, 3, 4, 11, 12
      4. state of resources created by an internal procedure exposed via output params. 
         cases: 015a
         - valid:   5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 2, 3, 4, 11, 12
      5. state of resources created by a function and exposed via output params. 
         cases: 015b
         - valid:   5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 2, 3, 4, 11, 12

III. procedure is ran non-persistent
   A. dynamic resources
      1. state of resources created by external program and saved in shared vars.
         cases: 017a
         - valid:   all
         - invalid: none
      2. state of resources created by an internal procedure exposed via shared vars. 
         cases: 017b
         - valid:   all
         - invalid: none
      3. state of resources created by a function and exposed via shared vars. 
         cases: 017c
         - valid:   all
         - invalid: none
   B. static resources
      1. state of resources created by external program and saved in shared vars.
         cases: 018a
         - valid:   5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 2, 3, 4, 11, 12
      2. state of resources created by an internal procedure exposed via shared vars. 
         cases: 018b
         - valid:   5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 2, 3, 4, 11, 12
      3. state of resources created by a function and exposed via shared vars. 
         cases: 018c
         - valid:   5, 6*, 7*, 8*, 9*, 10*, 13*, 14*, 15*, 16*, 17*
         - invalid: 1, 2, 3, 4, 11, 12
         
FINAL RULES:
- all dynamic resources and PROCEDURE resources survive the instantiating procedure
- the static resources (QUERY, TEMP-TABLE, BUFFER, widgets) get deleted when the instantiating procedure gets deleted
*/

def var how1 as log.
def var how2 as log.
def var how3 as log.
def var how4 as log.
def var how5 as log.
def var how6 as log.
def var how7 as log.
def var how8 as log.
def var how9 as log.
def var how10 as log.
def var how11 as log.
def var how12 as log.
def var how13 as log.
def var how14 as log.
def var how15 as log.
def var how16 as log.
def var how17 as log.

{create_resource_def_handle_vars.i &prefix="d" &shared="new shared" &var="var"}
{create_resource_def_handle_vars.i &prefix="cd" &shared="new shared" &var="var"}
{create_resource_def_handle_vars.i &prefix="fcd" &shared="new shared" &var="var"}
{create_resource_def_handle_vars.i &prefix="pd" &var="var" &shared=}
{create_resource_def_handle_vars.i &prefix="fpd" &var="var" &shared=}

{create_resource_def_handle_vars.i &prefix="s" &shared="new shared" &var="var"}
{create_resource_def_handle_vars.i &prefix="cs" &shared="new shared" &var="var"}
{create_resource_def_handle_vars.i &prefix="fcs" &shared="new shared" &var="var"}
{create_resource_def_handle_vars.i &prefix="ps" &var="var" &shared=}
{create_resource_def_handle_vars.i &prefix="fps" &var="var" &shared=}

def var pdh as handle.
def var psh as handle.

run create_resource_dynamic.p persistent set pdh(input no).
run create_res_dynamic in pdh({output_par_handles.i &prefix="pd" &as=}).
dynamic-function("create_res_dynamic_func" in pdh, {output_par_handles.i &prefix="fpd" &as=}).

run create_resource_static.p persistent set psh(input no).
run create_res_static in psh({output_par_handles.i &prefix="ps" &as=}).
dynamic-function("create_res_static_func" in psh, {output_par_handles.i &prefix="fps" &as=}).

{set_all_how.i &how=yes} 
{check_resources.i          &caz="001a" &prefix="d"}
{check_resource_inst_proc.i &caz="002a" &prefix="d" &how="no"}

{set_all_how.i &how=yes} 
{check_resources.i          &caz="001b" &prefix="cd"}
{check_resource_inst_proc.i &caz="002b" &prefix="cd" &how="no"}

{set_all_how.i &how=yes} 
{check_resources.i          &caz="001c" &prefix="fcd"}
{check_resource_inst_proc.i &caz="002c" &prefix="fcd" &how="no"}

{set_all_how.i &how=yes} 
{check_resources.i          &caz="003a" &prefix="pd"}
{check_resource_inst_proc.i &caz="004a" &prefix="pd" &how="no"}

{set_all_how.i &how=yes} 
{check_resources.i          &caz="003b" &prefix="fpd"}
{check_resource_inst_proc.i &caz="004b" &prefix="fpd" &how="no"}

{set_all_how.i &how=yes} 
{check_resources.i          &caz="005a" &prefix="s"}
{check_resource_inst_proc.i &caz="006a" &prefix="s" &how="no"}

{set_how.i &how1=no
           &how2=yes
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="005b" &prefix="cs"}
{check_resource_inst_proc.i &caz="006b" &prefix="cs" &how="no"}

{set_how.i &how1=no
           &how2=yes
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="005c" &prefix="fcs"}
{check_resource_inst_proc.i &caz="006c" &prefix="fcs" &how="no"}

{set_how.i &how1=no
           &how2=yes
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no 
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="007a" &prefix="ps"}
{check_resource_inst_proc.i &caz="008a" &prefix="ps" &how="no"}

{set_how.i &how1=no
           &how2=yes
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="007b" &prefix="fps"}
{check_resource_inst_proc.i &caz="008b" &prefix="fps" &how="no"}

delete object pdh.
delete object psh.

{set_all_how.i &how=yes} 
{check_resources.i          &caz="009a" &prefix="d"}
{check_resource_inst_proc.i &caz="010a" &prefix="d" &how="yes"}

{set_all_how.i &how=yes} 
{check_resources.i          &caz="009b" &prefix="cd"}
{check_resource_inst_proc.i &caz="010b" &prefix="cd" &how="yes"}

{set_all_how.i &how=yes} 
{check_resources.i          &caz="009c" &prefix="fcd"}
{check_resource_inst_proc.i &caz="010c" &prefix="fcd" &how="yes"}

{set_all_how.i &how=yes} 
{check_resources.i          &caz="011a" &prefix="pd"}
{check_resource_inst_proc.i &caz="012a" &prefix="pd" &how="yes"}

{set_all_how.i &how=yes} 
{check_resources.i          &caz="011b" &prefix="fpd"}
{check_resource_inst_proc.i &caz="012b" &prefix="fpd" &how="yes"}

{set_how.i &how1=no
           &how2=no
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="013a" &prefix="s"}
{check_resource_inst_proc.i &caz="014a" &prefix="s" &how="yes"}

{set_how.i &how1=no
           &how2=no
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="013b" &prefix="cs"}
{check_resource_inst_proc.i &caz="014b" &prefix="cs" &how="yes"}

{set_how.i &how1=no
           &how2=no
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="013c" &prefix="fcs"}
{check_resource_inst_proc.i &caz="014c" &prefix="fcs" &how="yes"}

{set_how.i &how1=no
           &how2=no
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="015a" &prefix="ps"}
{check_resource_inst_proc.i &caz="016a" &prefix="ps" &how="yes"}

{set_how.i &how1=no
           &how2=no
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no 
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="015b" &prefix="fps"}
{check_resource_inst_proc.i &caz="016b" &prefix="fps" &how="yes"}

run create_resource_dynamic.p(input yes).
run create_resource_static.p(input yes).

{set_all_how.i &how=yes}
{check_resources.i          &caz="017a" &prefix="d"}
{check_resource_inst_proc.i &caz="017a" &prefix="d" &how="yes"}

{set_all_how.i &how=yes}
{check_resources.i          &caz="017b" &prefix="cd"}
{check_resource_inst_proc.i &caz="017b" &prefix="cd" &how="yes"}

{set_all_how.i &how=yes}
{check_resources.i          &caz="017c" &prefix="fcd"}
{check_resource_inst_proc.i &caz="017c" &prefix="fcd" &how="yes"}

{set_how.i &how1=no
           &how2=no
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="018a" &prefix="s"}
{check_resource_inst_proc.i &caz="018a" &prefix="s" &how="yes"}

{set_how.i &how1=no
           &how2=no
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="018b" &prefix="cs"}
{check_resource_inst_proc.i &caz="018b" &prefix="cs" &how="yes"}

{set_how.i &how1=no
           &how2=no
           &how3=no
           &how4=no
           &how5=yes
           &how6=yes
           &how7=yes
           &how8=yes
           &how9=yes
           &how10=yes
           &how11=no
           &how12=no
           &how13=yes
           &how14=yes
           &how15=yes
           &how16=yes
           &how17=yes
} 
{check_resources.i          &caz="018c" &prefix="fcs"}
{check_resource_inst_proc.i &caz="018c" &prefix="fcs" &how="yes"}